import 'package:flutter/services.dart';
import 'package:shoplive_player/shoplive_player.dart';

import 'shoplive_player_platform_interface.dart';

/// An implementation of [ShoplivePlayerPlatform] that uses method channels.
class MethodChannelShoplivePlayer extends ShoplivePlayerPlatform {
  /// The method channel used to interact with the native platform.
  final methodChannel = const MethodChannel('shoplive_player');

  @override
  void setMixWithOthers({
    required bool isMixAudio,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_setMixWithOthers', <String, dynamic>{
      'isMixAudio': isMixAudio,
    });
  }

  @override
  void useCloseButton({
    required bool canUse,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_useCloseButton', <String, dynamic>{
      'canUse': canUse,
    });
  }

  @override
  void play({
    required ShopLivePlayerData data,
  }) async {
    return await methodChannel.invokeMethod<void>('player_play', <String, dynamic>{
      'campaignKey': data.campaignKey,
      'keepWindowStateOnPlayExecuted': data.keepWindowStateOnPlayExecuted,
      'referrer': data.referrer
    });
  }

  @override
  void close() async {
    return await methodChannel.invokeMethod<void>('player_close');
  }

  @override
  void setShareScheme({
    required String shareSchemeUrl,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_setShareScheme', <String, dynamic>{
      'shareSchemeUrl': shareSchemeUrl,
    });
  }

  @override
  void setEndpoint({
    required String? endpoint,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_setEndpoint', <String, dynamic>{
      'endpoint': endpoint,
    });
  }

  @override
  void setNextActionOnHandleNavigation({
    required ShopLiveActionType type,
  }) async {
    return await methodChannel.invokeMethod<void>(
        'player_setNextActionOnHandleNavigation', <String, dynamic>{
      'type': type.parseValue(),
    });
  }

  @override
  void setEnterPipModeOnBackPressed({
    required bool isEnterPipMode,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_setEnterPipModeOnBackPressed', <String, dynamic>{
      'isEnterPipMode': isEnterPipMode,
    });
  }

  @override
  void setMuteWhenPlayStart({
    required bool isMute,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_setMuteWhenPlayStart', <String, dynamic>{
      'isMute': isMute,
    });
  }

  @override
  void addParameter({
    required String key,
    required String? value,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_addParameter', <String, dynamic>{
      'key': key,
      'value': value,
    });
  }

  @override
  void removeParameter({
    required String key,
  }) async {
    return await methodChannel
        .invokeMethod<void>('player_removeParameter', <String, dynamic>{
      'key': key,
    });
  }
}
